from setuptools import setup, find_packages

setup(
    name="cloud_storage_slim",
    version="0.1.5",
    author="Ying Cai",
    author_email="i@caiying.me",
    description="A lightweight cloud storage package for popular cloud storage services.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/yingca1/cloud-storage-slim",
    packages=find_packages(),
    install_requires=[],
)
